@@ -5,6 +5,9 @@ import collections
 import itertools
 import six
 
+import logging
+logger = logging.getLogger(__name__)
+
 
 class ProtocolBase( collections.MutableMapping):
     __propinfo__ = {}
@@ -36,7 +39,8 @@ class ProtocolBase( collections.MutableMapping):
         return repr(self)
 
     def __repr__(self):
-        props = ["%s=%s" % (k, str(v)) for k, v in
+        inverter = dict((v, k) for k,v in self.__prop_names__.iteritems())
+        props = ["%s=%s" % (inverter[k], str(v)) for k, v in
                  itertools.chain(self._properties.iteritems(),
                                  self._extended_properties.iteritems())]
         return "<%s %s>" % (
@@ -44,22 +48,37 @@ class ProtocolBase( collections.MutableMapping):
             " ".join(props)
         )
 
+    @classmethod
+    def from_json(cls, jsonmsg):
+      import json
+      msg = json.loads(jsonmsg)
+      obj = cls(**msg)
+      obj.validate()
+      return obj
+
     def __init__(this, **props):
         this._extended_properties = dict()
-        this._properties = dict(zip(this.__prop_names__.keys(),
+        this._properties = dict(zip(this.__prop_names__.values(),
                                     [None for x in
                                      xrange(len(this.__prop_names__))]))
 
         for prop in props:
+            #try:
+            #    propname = this.__prop_names__[prop]
+            #except KeyError:
+            #    propname = prop
+
             try:
-                propname = this.__prop_names__[prop]
-            except KeyError:
-                propname = prop
+              logging.debug("Setting value for '{0}' to {1}"
+                            .format(prop, props[prop]))
+              setattr(this, prop, props[prop])
+            except validators.ValidationError as e:
+              import sys
+              raise type(e), type(e)(str(e) + " \nwhile setting '{0}' in {1}".format(
+                  prop, this.__class__.__name__)), sys.exc_info()[2]
 
-            setattr(this, propname, props[prop])
-
-        if len(props) > 0:
-            this.validate()
+        #if len(props) > 0:
+        #    this.validate()
 
     def __setattr__(self, name, val):
       if name.startswith("_"):
@@ -69,13 +88,14 @@ class ProtocolBase( collections.MutableMapping):
         # The property does special validation, so we actually need to
         # run its setter. We get it from the class definition and call
         # it directly. XXX Heinous.
-        prop = self.__class__.__dict__[name]
+        prop = self.__class__.__dict__[self.__prop_names__[name]]
         prop.fset(self, val)
       else:
         # This is an additional property of some kind
         if getattr(self, '__extensible__', None) is False:
           raise validators.ValidationError(
-              "Attempted to set unknown property '{0}', but 'additionalProperties' is false.")
+              "Attempted to set unknown property '{0}', "
+              "but 'additionalProperties' is false.".format(name))
         else:
           typ = getattr(self, '__extensible__', None)
           if typ is True:
@@ -84,7 +104,7 @@ class ProtocolBase( collections.MutableMapping):
             valtype = [k for k, t in self.__SCHEMA_TYPES__.iteritems()
                        if t is not None and isinstance(val, t)][0]
             val = MakeLiteral(name, valtype, val)
-          elif isinstance(typ, type) and issubclass(typ, LiteralValue):
+          elif isinstance(typ, type) and typ.isLiteralClass is True:
             val = typ(val)
           elif isinstance(typ, type) and issubclass(typ, ProtocolBase):
             val = typ(**val)
@@ -129,8 +149,10 @@ class ProtocolBase( collections.MutableMapping):
         return enc.encode(self)
 
     def validate(self):
+        propname = lambda x: self.__prop_names__[x]
         missing = [x for x in self.__required__
-                   if x not in self._properties or self._properties[x] is None]
+                   if propname(x) not in self._properties
+                   or self._properties[propname(x)] is None]
 
         if len(missing) > 0:
             raise validators.ValidationError(
@@ -143,7 +165,7 @@ class ProtocolBase( collections.MutableMapping):
 
             if isinstance(val, ProtocolBase):
                 val.validate()
-            elif isinstance(val, LiteralValue):
+            elif getattr(val, 'isLiteralClass', None) is True:
                 val.validate()
             elif isinstance(val, list):
                 for subval in val:
@@ -168,6 +190,8 @@ def MakeLiteral(name, typ, value, **properties):
 class LiteralValue(object):
   """Docstring for LiteralValue """
 
+  isLiteralClass = True
+
   def __init__(self, value, typ=None):
       """@todo: to be defined
 
@@ -214,6 +238,7 @@ class LiteralValue(object):
               else:
                   validator(paramval, self._value)
 
+
   def __cmp__(self, other):
     if isinstance(other, six.integer_types):
       return cmp(int(self), other)
@@ -260,7 +285,14 @@ class ClassBuilder(object):
 
         return pp
 
-    def construct(self, uri, clsdata, parent=(ProtocolBase,)):
+    def construct(self, uri, *args, **kw):
+        """ Wrapper to debug things """
+        logger.debug("Constructing {0}".format(uri))
+        ret = self._construct(uri, *args, **kw)
+        logger.debug("Constructed {0}".format(ret))
+        return ret
+
+    def _construct(self, uri, clsdata, parent=(ProtocolBase,)):
         if 'oneOf' in clsdata:
             potential_parents = self.resolve_classes(clsdata['oneOf'])
 
@@ -273,6 +305,8 @@ class ClassBuilder(object):
                 else:
                     raise Exception("Don't know how to deal with this")
 
+            return self.resolved[uri]
+
         elif 'anyOf' in clsdata:
             raise NotImplementedError(
                 "anyOf is not supported as bare property")
@@ -295,15 +329,24 @@ class ClassBuilder(object):
 
         elif '$ref' in clsdata:
 
-            if uri in self.resolved:
-                return self.resolved[uri]
+            if 'type' in clsdata and issubclass(clsdata['type'], ProtocolBase):
+                # It's possible that this reference was already resolved, in which
+                # case it will have its type parameter set
+                logging.debug("Using previously resolved type "
+                              "(with different URI) for {0}".format(uri))
+                self.resolved[uri] = clsdata['type']
+            elif uri in self.resolved:
+                logging.debug("Using previously resolved object for {0}".format(uri))
             else:
+                logging.debug("Resolving object for {0}".format(uri))
+
                 with self.resolver.resolving(uri) as resolved:
                     self.resolved[uri] = self._build_object(
                         uri,
                         resolved,
                         parent)
-                return self.resolved[uri]
+
+            return self.resolved[uri]
 
         elif 'array' in clsdata and 'items' in clsdata:
             self.resolved[uri] = self._build_object(
@@ -324,10 +367,18 @@ class ClassBuilder(object):
                 uri,
                 clsdata)
             return self.resolved[uri]
+        elif 'enum' in clsdata:
+            obj = self._build_literal(uri, clsdata)
+            self.resolved[uri] = obj
+            return obj
+
+        elif 'type' in clsdata and issubclass(clsdata['type'], ProtocolBase):
+            self.resolved[uri] = clsdata.get('type')
+            return self.resolved[uri]
         else:
             raise NotImplementedError(
-                "Unable to parse schema object with "
-                "no type and no reference")
+                "Unable to parse schema object '{0}' with "
+                "no type and no reference".format(clsdata))
 
     def _build_literal(self, nm, clsdata):
       """@todo: Docstring for _build_literal
@@ -344,6 +395,7 @@ class ClassBuilder(object):
       return cls
 
     def _build_object(self, nm, clsdata, parents):
+        logger.debug("Building object {0}".format(nm))
 
         props = {}
 
@@ -536,7 +588,7 @@ def make_property(prop, info, desc=""):
             instance = info['validator'](val)
             val = instance.validate()
 
-        elif issubclass(info['type'], LiteralValue):
+        elif getattr(info['type'], 'isLiteralClass', False) is True:
             if not isinstance(val, info['type']):
                 val = info['type'](val)
 
